/*
 * MUST (Marmot Umpire Scalable Tool)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2011-2012
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2011-2012
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file Wfg.cpp
 *       @see must::Wfg.
 *
 *  @date 12.08.2011
 *  @author Tobias Hilbrich, Mathias Korepkat, Joachim Protze
 */

#include "Wfg.h"
#include "wfglib.h"

using namespace must;

//=============================
// NodeInfo
//=============================
NodeInfo::NodeInfo ()
: type (ARC_AND),
  outArcs ()
{
    //Nothing to do
}

//=============================
// Wfg
//=============================
Wfg::Wfg (void)
: myWfg (),
  myMaxNode (-1)
{
    //Nothing to do
}

//=============================
// ~Wfg
//=============================
Wfg::~Wfg (void)
{
    myWfg.clear();
}

//=============================
// addArc
//=============================
bool Wfg::addArc (int from, int to, ArcType type)
{
    if (from > myMaxNode)
        myMaxNode = from;
    if (to > myMaxNode)
        myMaxNode = to;

    if (myWfg.find (from) != myWfg.end())
    {
        if (myWfg[from].type !=type)
            return false;
    }
    else
    {
        myWfg[from].type = type;
    }

    myWfg[from].outArcs.push_back (to);
    return true;
}

//=============================
// detectDeadlock
//=============================
bool Wfg::detectDeadlock (bool *outHasDeadlock, std::list<int> *outDeadlockedNodes)
{
    /**
     * @todo the myMaxNode+1 thing may be very painfull and inefficient!
     * Mapping used node ids to continous ints would be smarter to minimize
     * the WFG size, but lets see whether this becomes an issue at some
     * point.
     */

    if (myMaxNode == -1)
        return true;

    //1)== Init
    if (wfg_initialize (myMaxNode+1) != WFG_SUCCESS)
        return false;

    //2)== Add arcs
    std::map<int, NodeInfo>::iterator fromIter;
    for (fromIter = myWfg.begin();fromIter != myWfg.end(); fromIter++)
    {
        int from = fromIter->first;

        WFG_ARC_TYPE type = WFG_ARC_AND;
        if (fromIter->second.type == ARC_OR)
            type = WFG_ARC_OR;

        std::list<int>::iterator toIter;
        for (toIter = fromIter->second.outArcs.begin(); toIter != fromIter->second.outArcs.end(); toIter++)
        {
            int to = *toIter;

            if (wfg_add_arc (from, to, type) != WFG_SUCCESS)
                return false;
        }
    }

    //3)== Detect
    WFG_RETURN ret = wfg_deadlock_check ();
    if (ret == WFG_ERROR) return false;

    //4)== Prepare outputs
    if (ret == WFG_DEADLOCK)
    {
        int *pNodes = new int [myMaxNode+1];
        int count;
        if (wfg_get_deadlocked_nodes (&count, pNodes) != WFG_SUCCESS)
            return false;

        if (outDeadlockedNodes)
        {
            for (int i = 0; i < count; i++)
                outDeadlockedNodes->push_back(pNodes[i]);
        }

        if (outHasDeadlock)
            *outHasDeadlock = true;

        if (pNodes) delete [] pNodes;
    }
    else
    {
        if (outHasDeadlock)
            *outHasDeadlock = false;
    }

    //5)== Clean up
    if (wfg_finalize () != WFG_SUCCESS)
        return false;

    return true;
}

/*EOF*/
